@@ -413,8 +413,8 @@ impl<'a, 'de: 'a, T: DeRecord<'de>> Deserializer<'de>
         self,
         visitor: V,
     ) -> Result<V::Value, Self::Error> {
-        self.next_field()
-            .and_then(|f| visitor.visit_byte_buf(f.as_bytes().to_vec()))
+        self.next_field_bytes()
+            .and_then(|f| visitor.visit_byte_buf(f.to_vec()))
     }
 
     fn deserialize_option<V: Visitor<'de>>(
@@ -752,7 +752,7 @@ mod tests {
     use std::collections::HashMap;
 
     use serde::de::DeserializeOwned;
-    use serde_bytes::ByteBuf;
+    use serde_bytes::{self, ByteBuf};
 
     use byte_record::ByteRecord;
     use error::Error;
@@ -775,6 +775,10 @@ mod tests {
         deserialize_string_record(&record, Some(&headers))
     }
 
+    fn b<'a, T: AsRef<[u8]> + ?Sized>(bytes: &'a T) -> &'a [u8] {
+        bytes.as_ref()
+    }
+
     #[test]
     fn with_header() {
         #[derive(Deserialize, Debug, PartialEq)]
@@ -1129,4 +1133,59 @@ mod tests {
             .collect();
         assert_eq!(got, expected);
     }
+
+    #[test]
+    fn flatten() {
+        #[derive(Deserialize, Debug, PartialEq)]
+        struct Input {
+            x: f64,
+            y: f64,
+        }
+
+        #[derive(Deserialize, Debug, PartialEq)]
+        struct Properties {
+            prop1: f64,
+            prop2: f64,
+        }
+
+        #[derive(Deserialize, Debug, PartialEq)]
+        struct Row {
+            #[serde(flatten)]
+            input: Input,
+            #[serde(flatten)]
+            properties: Properties,
+        }
+
+        let header = StringRecord::from(vec!["x", "y", "prop1", "prop2"]);
+        let record = StringRecord::from(vec!["1", "2", "3", "4"]);
+        let got: Row = record.deserialize(Some(&header)).unwrap();
+        assert_eq!(got, Row {
+            input: Input { x: 1.0, y: 2.0 },
+            properties: Properties { prop1: 3.0, prop2: 4.0 },
+        });
+    }
+
+    #[test]
+    fn partially_invalid_utf8() {
+        #[derive(Debug, Deserialize, PartialEq)]
+        struct Row {
+            h1: String,
+            #[serde(with = "serde_bytes")]
+            h2: Vec<u8>,
+            h3: String,
+        }
+
+        let headers = ByteRecord::from(vec![b"h1", b"h2", b"h3"]);
+        let record = ByteRecord::from(vec![
+            b(b"baz"), b(b"foo\xFFbar"), b(b"quux"),
+        ]);
+        let got: Row = deserialize_byte_record(
+            &record, Some(&headers),
+        ).unwrap();
+        assert_eq!(got, Row {
+            h1: "baz".to_string(),
+            h2: b"foo\xFFbar".to_vec(),
+            h3: "quux".to_string(),
+        });
+    }
 }
